﻿'use strict';

ko.bindingHandlers.select2 = {
    after: ["options", "value"],
    init: function (el, valueAccessor) {
        $(el).select2(ko.unwrap(valueAccessor()));
        ko.utils.domNodeDisposal.addDisposeCallback(el, function () {
            $(el).select2('destroy');
        });
    },
    update: function (el, valueAccessor, allBindingsAccessor) {
        var allBindings = allBindingsAccessor();
        var select2 = $(el).data("select2");
        if ("value" in allBindings) {
            var newValue = "" + ko.unwrap(allBindings.value);
            if ((allBindings.select2.multiple || el.multiple) && newValue.constructor !== Array) {
                select2.val([newValue.split(",")]);
            }
            else {
                select2.val(newValue ? [newValue] : '');
            }
        }
    }
};

ko.bindingHandlers.tooltipbs = {
    init: function (element, valueAccessor) {
        //ko.utils.extend(options, local);
        //bs tooltip nereaguje na predavani title do objektu options https://getbootstrap.com/docs/3.3/javascript/#tooltips
        $(element).attr('title', ko.utils.unwrapObservable(valueAccessor()));
        $(element).tooltip({ trigger: 'hover' });
        ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
            $(element).tooltip("destroy");
        });
    },
    update: function (element, valueAccessor) {
        $(element).attr('title', ko.utils.unwrapObservable(valueAccessor()));
    }
};

ko.bindingHandlers.booleanValue = {
    init: function (element, valueAccessor) {
        var observable = valueAccessor(),
            interceptor = ko.computed({
                read: function () {
                    return observable().toString();
                },
                write: function (newValue) {
                    observable(newValue === "true");
                }
            });

        ko.applyBindingsToNode(element, { value: interceptor });
    }
};

ko.bindingHandlers.executeOnEnter = {
    init: function (element, valueAccessor) {
        var callback = valueAccessor();
        $(element).keypress(function (e) {
            var keyCode = e.which || e.keyCode;
            if (keyCode === 13 && !(e.ctrlKey || e.shiftKey)) {
                callback.call(element.value);
                return false;
            }
            else if ((keyCode === 13 || keyCode === 10) && (e.ctrlKey || e.shiftKey)) {
                var val = element.value;
                if (typeof element.selectionStart === "number" && typeof element.selectionEnd === "number") {
                    var start = element.selectionStart;
                    element.value = val.slice(0, start) + "\n" + val.slice(element.selectionEnd);
                    element.selectionStart = element.selectionEnd = start + 1;
                } else if (document.selection && document.selection.createRange) {
                    element.focus();
                    var range = document.selection.createRange();
                    range.text = "\r\n";
                    range.collapse(false);
                    range.select();
                }

                let lineHeight = element.clientHeight / element.rows;
                element.scrollTop = element.scrollTop + lineHeight;
            }

            return true;
        });
    }
};

ko.bindingHandlers.resizeOnKeyup = {
    init: function (element, valueAccessor) {
        var val = valueAccessor();
        if (val) {
            element.addEventListener('keyup', function () {
                this.style.height = 0;
                this.style.height = this.scrollHeight - 17 + 'px';
            }, false);
        }
    }
};

function globalIsElementInView(element, fullyInView) {
    var pageTop = $(window).scrollTop();
    var pageBottom = pageTop + $(window).height();
    var elementTop = $(element).offset().top;
    var elementBottom = elementTop + $(element).height();

    if (fullyInView === true) {
        return ((pageTop < elementTop) && (pageBottom > elementBottom));
    } else {
        return ((elementTop <= pageBottom) && (elementBottom >= pageTop));
    }
}

/** Je závislý na typu itemů. Není to obecný custom binding handler. */
ko.bindingHandlers.scrollDown = {
    init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        let scrollDownObserver = new IntersectionObserver(function (entries, observer) {
            if (entries.length >= 1) {
                let entry = entries[entries.length - 1];
                if (entry.isIntersecting) {
                    var scrollHeight = $(element)[0].scrollHeight;
                    if (scrollHeight > 0) {
                        $(element).parent().scrollTop(scrollHeight);
                        observer.unobserve(element);
                    }
                }
            }
        }, {
                root: element.parentElement,
                rootMargin: '0px',
                threshold: 0.0
            });

        scrollDownObserver.observe(element);
        ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
            scrollDownObserver.unobserve(element);
        });
    },
    update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        var arr = ko.utils.unwrapObservable(valueAccessor());
        if (Array.isArray(arr) && arr.length > 0) {
            if (arr[0].PostLoaded) {
                let i = 0;
                do {
                    arr[i].PostLoaded = false;
                } while (arr[++i].PostLoaded);
            }
            else {
                setTimeout(function () {
                    //if (arr[arr.length - 1].Type !== chatUtils.ChatMessageTypeEnum.COMMON_INCOMPLETE || globalIsElementInView($("li:last-child.chat-message-container, li:last-child.chat-event-message-container", element), true)) {
                        var scrollHeight = $(element)[0].scrollHeight;
                        if (scrollHeight > 0) {
                            $(element).parent().scrollTop(scrollHeight);
                        }
                    //}
                }, 0);
            }
        }
    }
};


ko.bindingHandlers.onScrollEdge = {
    init: function (element, valueAccessor) {
        let listElement = element.getElementsByTagName('ul')[0];
        if (!listElement)
            return;

        element.addEventListener('scroll', function (event) {
            if (event.target.scrollTop === 0) {
                let handlerObj = valueAccessor();
                if ("topHandler" in handlerObj) {
                    handlerObj.topHandler();
                }
            }
            else if (event.target.scrollTop + event.target.offsetHeight === event.target.scrollHeight) {
                let handlerObj = valueAccessor();
                if ("bottomHandler" in handlerObj) {
                    handlerObj.bottomHandler();
                }
            }
        }, false);

        let latestScrollHeight = 0;

        let scrollMutationObserver = new MutationObserver(function (mutations) {
            let newList = false;
            if (mutations.length > 0 && mutations[0].addedNodes.length > 0 && (mutations[0].addedNodes[0].previousElementSibling === null || mutations[0].addedNodes[0].previousSibling === null)) { //node byl pridan na zacatek seznamu
                let lastMutation = mutations[mutations.length - 1];
                if (lastMutation.addedNodes[0].nextElementSibling !== null) {
                    let $parent = $(listElement).parent();
                    $parent.css("overflow-y", "hidden");
                    $parent.scrollTop(listElement.scrollHeight - latestScrollHeight);
                    $parent.css("overflow-y", "auto");
                }
                else
                    newList = true;
            }

            latestScrollHeight = listElement.scrollHeight;
            let handlerObj = valueAccessor();
            if ('expressionToSearch' in handlerObj && handlerObj.expressionToSearch) {
                $(listElement).find('li div.messenger-common-message-body, li div.messenger-attachments-message-body > div > a').highlight(handlerObj.expressionToSearch, false);
                if (newList) {
                    let $messageOccurrence = $(listElement).find('li:eq(' + handlerObj.lastMessageOccurrenceIndexInCluster + ')');
                    $messageOccurrence.find("div .highlight").css("background-color", "orange");
                    let $parent = $(listElement).parent();
                    $parent.scrollTop($messageOccurrence[0].offsetTop);
                }
            }
        });

        scrollMutationObserver.observe(listElement, {
            childList: true
        });

        ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
            scrollMutationObserver.disconnect();
        });
    }
};



ko.bindingHandlers.triggerWhenVisible = {
    init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        let visibilityObserver = new IntersectionObserver(function (entries, observer) {
            if (entries.length >= 1) {
                let entry = entries[entries.length - 1];
                if (entry.isIntersecting) {
                    valueAccessor()(true);
                    observer.unobserve(element);
                }
            }
        }, {
                root: null,//element.parentElement,
                rootMargin: '0px',
                threshold: 0.0
            });

        visibilityObserver.observe(element);
        ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
            visibilityObserver.unobserve(element);
        });
    }
};


ko.bindingHandlers.playerSwitch = {
    update: function (element, valueAccessor) {
        var play = ko.unwrap(valueAccessor());
        if (play) {
            if (element.paused)
                element.play();
        }
        else {
            if (!element.paused) {
                element.pause();
                element.currentTime = 0;
            }
        }
    }
};

ko.bindingHandlers.srcObject = {
    update: function (element, valueAccessor) {
        element.srcObject = ko.unwrap(valueAccessor());
    }
};

ko.bindingHandlers.plupload = {
    init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        function guid() {
            function s4() {
                return Math.floor((1 + Math.random()) * 0x10000)
                    .toString(16)
                    .substring(1);
            }
            return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
                s4() + '-' + s4() + s4() + s4();
        }

        var randomId = Math.floor(Math.random() * 1000000).toString();
        element.id = randomId; //predat pluploadu primo element nefunguje (podle dokumentace by melo)

        var uploader = new plupload.Uploader({
            runtimes: 'html5,flash,silverlight',
            url: RQ_URL + "/Chat/UploadAttachmentAjax",
            browse_button: randomId, //predat pluploadu primo element nefunguje (podle dokumentace by melo)
            max_file_size: '500mb',
            chunk_size: '1mb',
            unique_names: false,
            multiple_queues: false,
            multi_selection: false,
            multipart_params: { identifierForAsyncAttachments: guid() },
            max_file_count: 1,
            flash_swf_url: RQ_URL + "/Scripts/plupload/Moxie.swf",
            silverlight_xap_url: RQ_URL + "/Scripts/plupload/Moxie.xap"
        });

        var assignUploader = function (uploader) { this.initUploader(uploader); };
        assignUploader.call(bindingContext.$data, uploader);
    }
};